@@ -50,7 +50,7 @@
  *   [ -tabs [tab1 tab2 tab3 ... ] ]     (tabn = 1-160)
  *   [ -clrtabs [ tab1 tab2 tab3 ... ]   (tabn = 1-160)
  *   [ -regtabs [1-160] ]
- *   [ -blank [0-60] ]
+ *   [ -blank [0-60|force|poke|] ]
  *   [ -dump   [1-NR_CONS ] ]
  *   [ -append [1-NR_CONS ] ]
  *   [ -file dumpfilename ]
@@ -102,20 +102,24 @@
 #define NCURSES_CONST const	/* define before including term.h */
 #endif
 #include <term.h>
-#if NCH
+
+#ifdef HAVE_NCURSES_H
 #include <ncurses.h>
-#else
-#include <curses.h>
+#elif defined(HAVE_NCURSES_NCURSES_H)
+#include <ncurses/ncurses.h>
 #endif
+
+#include <sys/param.h>		/* for MAXPATHLEN */
 #include <sys/ioctl.h>
 #include <sys/time.h>
-#include "nls.h"
-
-#ifndef TCGETS
-/* TCGETS is either defined in termios.h, or here: */
-#include <asm/ioctls.h>
+#ifdef HAVE_LINUX_TIOCL_H
+#include <linux/tiocl.h>
 #endif
 
+#include "c.h"
+#include "xalloc.h"
+#include "nls.h"
+
 #if __GNU_LIBRARY__ < 5
 #ifndef __alpha__
 # include <linux/unistd.h>
@@ -129,12 +133,6 @@ extern int klogctl(int type, char *buf, int len);
 
 /* Constants. */
 
-/* General constants. */
-#ifndef TRUE
-#define TRUE  1
-#define FALSE 0
-#endif
-
 /* Keyboard types. */
 #define PC	 0
 #define OLIVETTI 1
@@ -153,6 +151,19 @@ extern int klogctl(int type, char *buf, int len);
 #define GREY	8
 #define DEFAULT 9
 
+/* Blank commands */
+#define BLANKSCREEN	-1
+#define UNBLANKSCREEN	-2
+#define BLANKEDSCREEN	-3
+
+/* <linux/tiocl.h> fallback */
+#ifndef TIOCL_BLANKSCREEN
+# define TIOCL_UNBLANKSCREEN	4       /* unblank screen */
+# define TIOCL_SETVESABLANK	10      /* set vesa blanking mode */
+# define TIOCL_BLANKSCREEN	14	/* keep screen blank even if a key is pressed */
+# define TIOCL_BLANKEDSCREEN	15	/* return which vt was blanked */
+#endif
+
 /* Control sequences. */
 #define ESC "\033"
 #define DCS "\033P"
@@ -184,7 +195,7 @@ int opt_cl_all;			/* Clear all or rest. */
 int opt_bl_min;			/* Blank screen. */
 int opt_blength_l;
 int opt_bfreq_f;
-int opt_sn_num = 0;		/* Snap screen. */
+int opt_sn_num;			/* Snap screen. */
 int opt_st_attr;
 int opt_rt_len;			/* regular tab length */
 int opt_tb_array[161];		/* Array for tab list */
@@ -290,9 +301,9 @@ par_color(int argc, char **argv, int *option, int *opt_color, int *bad_arg) {
 			*opt_color = DEFAULT;
 		else if (isdigit(argv[0][0]))
 			*opt_color = atoi(argv[0]);
-		else 
+		else
 			*bad_arg = TRUE;
-    
+
 		if(*opt_color < 0 || *opt_color > 9 || *opt_color == 8)
 			*bad_arg = TRUE;
 	}
@@ -347,7 +358,7 @@ par_color2(int argc, char **argv, int *option, int *opt_color, int *bad_arg) {
 			*opt_color |= WHITE;
 		else if (isdigit(argv[argc-1][0]))
 			*opt_color = atoi(argv[argc-1]);
-		else    
+		else
 			*bad_arg = TRUE;
 		if(*opt_color < 0 || *opt_color > 15)
 			*bad_arg = TRUE;
@@ -393,11 +404,17 @@ parse_blank(int argc, char **argv, int *option, int *opt_all, int *bad_arg) {
 		*bad_arg = TRUE;
 	*option = TRUE;
 	if (argc == 1) {
-		*opt_all = atoi(argv[0]);
-		if ((*opt_all > 60) || (*opt_all < 0))
-			*bad_arg = TRUE;
+		if (!strcmp(argv[0], "force"))
+			*opt_all = BLANKSCREEN;
+		else if (!strcmp(argv[0], "poke"))
+			*opt_all = UNBLANKSCREEN;
+		else {
+			*opt_all = atoi(argv[0]);
+			if ((*opt_all > 60) || (*opt_all < 0))
+				*bad_arg = TRUE;
+		}
 	} else {
-		*opt_all = 0;
+		*opt_all = BLANKEDSCREEN;
 	}
 }
 
@@ -781,7 +798,7 @@ usage(char *prog_name) {
 	fprintf(stderr, _("  [ -tabs [ tab1 tab2 tab3 ... ] ]      (tabn = 1-160)\n"));
 	fprintf(stderr, _("  [ -clrtabs [ tab1 tab2 tab3 ... ] ]   (tabn = 1-160)\n"));
 	fprintf(stderr, _("  [ -regtabs [1-160] ]\n"));
-	fprintf(stderr, _("  [ -blank [0-60] ]\n"));
+	fprintf(stderr, _("  [ -blank [0-60|force|poke] ]\n"));
 	fprintf(stderr, _("  [ -dump   [1-NR_CONSOLES] ]\n"));
 	fprintf(stderr, _("  [ -append [1-NR_CONSOLES] ]\n"));
 	fprintf(stderr, _("  [ -file dumpfilename ]\n"));
@@ -1036,13 +1053,32 @@ perform_sequence(int vcterm) {
 	}
 
 	/* -blank [0-60]. */
-	if (opt_blank && vcterm) 
-		printf("\033[9;%d]", opt_bl_min);
-    
+	if (opt_blank && vcterm) {
+		if (opt_bl_min >= 0)
+			printf("\033[9;%d]", opt_bl_min);
+		else if (opt_bl_min == BLANKSCREEN) {
+			char ioctlarg = TIOCL_BLANKSCREEN;
+			if (ioctl(0,TIOCLINUX,&ioctlarg))
+				fprintf(stderr,_("cannot force blank\n"));
+		} else if (opt_bl_min == UNBLANKSCREEN) {
+			char ioctlarg = TIOCL_UNBLANKSCREEN;
+			if (ioctl(0,TIOCLINUX,&ioctlarg))
+				fprintf(stderr,_("cannot force unblank\n"));
+		} else if (opt_bl_min == BLANKEDSCREEN) {
+			char ioctlarg = TIOCL_BLANKEDSCREEN;
+			int ret;
+			ret = ioctl(0,TIOCLINUX,&ioctlarg);
+			if (ret < 0)
+				fprintf(stderr,_("cannot get blank status\n"));
+			else
+				printf("%d\n",ret);
+		}
+	}
+
 	/* -powersave [on|vsync|hsync|powerdown|off] (console) */
 	if (opt_powersave) {
 		char ioctlarg[2];
-		ioctlarg[0] = 10;	/* powersave */
+		ioctlarg[0] = TIOCL_SETVESABLANK;
 		ioctlarg[1] = opt_ps_mode;
 		if (ioctl(0,TIOCLINUX,ioctlarg))
 			fprintf(stderr,_("cannot (un)set powersave mode\n"));
@@ -1084,7 +1120,7 @@ perform_sequence(int vcterm) {
 			result = klogctl(6, NULL, 0);
 
 		if (result != 0)
-			printf(_("klogctl error: %s\n"), strerror(result));
+			printf(_("klogctl error: %s\n"), strerror(errno));
 	}
 
 	/* -msglevel [0-8] */
@@ -1092,14 +1128,14 @@ perform_sequence(int vcterm) {
 		/* 8 -- Set level of messages printed to console */
 		result = klogctl(8, NULL, opt_msglevel_num);
 		if (result != 0)
-			printf(_("klogctl error: %s\n"), strerror(result));
+			printf(_("klogctl error: %s\n"), strerror(errno));
 	}
 
 	/* -blength [0-2000] */
 	if (opt_blength && vcterm) {
 		printf("\033[11;%d]", opt_blength_l);
 	}
-  
+
 	/* -bfreq freqnumber */
 	if (opt_bfreq && vcterm) {
 		printf("\033[10;%d]", opt_bfreq_f);
@@ -1109,7 +1145,6 @@ perform_sequence(int vcterm) {
 
 static void
 screendump(int vcnum, FILE *F) {
-#include <sys/param.h>
     char infile[MAXPATHLEN];
     unsigned char header[4];
     unsigned int rows, cols;
@@ -1117,23 +1152,36 @@ screendump(int vcnum, FILE *F) {
     char *inbuf, *outbuf, *p, *q;
 
     sprintf(infile, "/dev/vcsa%d", vcnum);
-    fd = open(infile, 0);
-    if (fd < 0 || read(fd, header, 4) != 4)
-      goto try_ioctl;
+    fd = open(infile, O_RDONLY);
+    if (fd < 0 && vcnum == 0) {
+	/* vcsa0 is often called vcsa */
+	sprintf(infile, "/dev/vcsa");
+	fd = open(infile, O_RDONLY);
+    }
+    if (fd < 0) {
+	/* try devfs name - for zero vcnum just /dev/vcc/a */
+	/* some gcc's warn for %.u - add 0 */
+	sprintf(infile, "/dev/vcc/a%.0u", vcnum);
+	fd = open(infile, O_RDONLY);
+    }
+    if (fd < 0) {
+	sprintf(infile, "/dev/vcsa%d", vcnum);
+	goto read_error;
+    }
+    if (read(fd, header, 4) != 4)
+	goto read_error;
     rows = header[0];
     cols = header[1];
     if (rows * cols == 0)
-      goto try_ioctl;
+        goto read_error;
     inbuf = malloc(rows*cols*2);
     outbuf = malloc(rows*(cols+1));
     if(!inbuf || !outbuf) {
 	fputs(_("Out of memory"), stderr);
-	exit(1);
-    }
-    if (read(fd, inbuf, rows*cols*2) != rows*cols*2) {
-	fprintf(stderr, _("Error reading %s\n"), infile);
-	exit(1);
+	goto error;
     }
+    if (read(fd, inbuf, rows*cols*2) != rows*cols*2)
+	    goto read_error;
     p = inbuf;
     q = outbuf;
     for(i=0; i<rows; i++) {
@@ -1147,36 +1195,17 @@ screendump(int vcnum, FILE *F) {
     }
     if (fwrite(outbuf, 1, q-outbuf, F) != q-outbuf) {
 	fprintf(stderr, _("Error writing screendump\n"));
-	exit(1);
+	goto error;
     }
+    close(fd);
     return;
 
-try_ioctl:
-    {
-#define NUM_COLS 160
-#define NUM_ROWS 75
-	char buf[NUM_COLS+1];
-	unsigned char screenbuf[NUM_ROWS*NUM_COLS];
-	screenbuf[0] = 0;
-	screenbuf[1] = (unsigned char) vcnum;
-	if (ioctl(0,TIOCLINUX,screenbuf) < 0) {
-	    fprintf(stderr,_("couldn't read %s, and cannot ioctl dump\n"),
-		    infile);
-	    exit(1);
-	}
-	rows = screenbuf[0];
-	cols = screenbuf[1];
-
-	for (i=0; i<rows; i++) {
-	    strncpy(buf, screenbuf+2+(cols*i), cols);
-	    buf[cols] = '\0';
-	    j = cols;
-	    while (--j && (buf[j] == ' '))
-	      buf[j] = '\0';
-	    fputs(buf,F);
-	    fputc('\n',F); 
-	}
-    }
+read_error:
+    fprintf(stderr, _("Couldn't read %s\n"), infile);
+error:
+    if (fd >= 0)
+	    close(fd);
+    exit(1);
 }
 
 int
@@ -1189,7 +1218,7 @@ main(int argc, char **argv) {
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-  
+
 	if (argc < 2)
 		bad_arg = TRUE;
 
